#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/poll.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <linux/i2c.h>
#include <linux/i2c-dev.h>

#include "my_i2c.h"
#include "adc.h"

void i2c_close();
void i2c_read(int delay, const unsigned int flag);

static struct struct_i2c_comm *i2c;


/* struct_i2c_comm 객체 initialize */
struct struct_i2c_comm *i2c_open()
{
	i2c = (struct struct_i2c_comm *)calloc(sizeof(struct struct_i2c_comm), 1);
	i2c->dev_fd = open(I2C_DEV, O_RDWR);

	i2c->close = i2c_close;
	i2c->read = i2c_read;

	return i2c;
}

/* struct_i2c_comm 소멸 */
void i2c_close()
{
	close(i2c->dev_fd);
	free(i2c);
}

/* i2c에 인자로들어온 addr주소의 adc값을 i2c_comm_struct->read_buf에 읽어온다 */
static int inline __read_i2c(unsigned char addr, int num)
{
	struct i2c_msg msg[2];
	struct i2c_rdwr_ioctl_data rdwr;
	int ret = num;
//	int buf_sel = (addr == COMMD_1)? 0 : (addr == COMMD_2)? 1 : (addr == COMMD_3)? 2 : 0;
	unsigned char buf[2];

	msg[0].flags = 0; // write address
	msg[0].addr = SLAVE_ADDR; //acd 장치addr
	msg[0].buf = &addr; // adc에 물려있는 센서 addr
	msg[0].len = 1;

	msg[1].flags = I2C_M_RD; // read data
	msg[1].addr = SLAVE_ADDR;
	switch (addr) {
	case COMMD_1:
		msg[1].buf = i2c->read_buf.acc[0];
		break;
	case COMMD_2:
		msg[1].buf = i2c->read_buf.acc[1];
		break;
	case COMMD_3:
		msg[1].buf = i2c->read_buf.acc[2];
		break;
	case COMMD_4:
		msg[1].buf = i2c->read_buf.piezo;
		break;
	}
//	msg[1].buf = i2c->read_buf.acc[buf_sel]; // 값을 읽어와서 저장할 버퍼
//	msg[1].buf = buf;
	msg[1].len = num; // 읽을바이트

	rdwr.msgs = &msg[0]; // 패킷 시작주소
	rdwr.nmsgs = 2; // 패킷갯수

	if ((ret = ioctl (i2c->dev_fd, I2C_RDWR, &rdwr)) < 0) {
		perror("ioctl(_read_acc) : read");
		return ret;
	}

	return num;
}

/* 가속도센서의 x, y, z 데이터를 읽어온다 */
static void _read_acc(int delay)
{
	__read_i2c(COMMD_1, 2);
	__read_i2c(COMMD_2, 2);
	__read_i2c(COMMD_3, 2);
	usleep(delay);
}

/* piezo센서의 값을 읽어온다 */
static void _read_piezo(int delay)
{
	__read_i2c(COMMD_4, 2);
	usleep(delay);
}

/* i2c에서 플래그값에따라 적절한 센서를 연결 */
void i2c_read(int delay, const unsigned int flag)
{
	void (*sel_func)(int) = NULL;

	switch (flag & ADC_SEL_MASK) {
		case ADC_ACCEL:
			sel_func = _read_acc;
			break;
		case ADC_PIEZO:
			sel_func = _read_piezo;
			break;
		default:
	}
	if (sel_func) 
		sel_func(delay);
}

