#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/ioctl.h>

#include "blue.h"

#define DEBUG

int bt_mode_data();
int bt_mode_at();
void bt_write_at(char *src, int count);
void bt_read_at(char *src);
static int bt_ato(unsigned int addr);

void bt_atcheck();
int bt_btinfo();
void bt_atz();
void bt_scan();
int bt_concheck();
void bt_send(struct blue_packet *btp, void *data, size_t size);
int bt_mode_data();
int bt_mode_at();

static int serial_fd;

/* BTUART 사용을 위한 초기화 함수 */
void bt_init()
{
	serial_fd = serial_init(B38400);
	serial_flush();
}

/* 블루투스 종료 함수 */
void bt_exit()
{
	serial_exit();
}

void bt_server_init()
{
	serial_set(serial_fd, 0);
	serial_flush();
}

/* 블루투스를 at명령어 모드에서 데이터모드로 변경한다 */
int bt_mode_data()
{
	int modemctl;

	_set_dtr();
	usleep(20000);
	printf("bt_mode_data ok!\n");

/*	while (1) {
		modemctl = serial_get_info();
		if ((modemctl & TIOCM_DSR) == TIOCM_DSR) {
#ifdef DEBUG
			printf("bt_mode_data ok!\n");
#endif
			return 1;
		}
		_set_dtr();
		usleep(10000);
	}*/
	return 0;
}

/* 블루투스를 데이터모드에서 명령어모드로 전환 */
int bt_mode_at()
{
	int modemctl;

	while (1) {
		_clear_dtr();
		usleep(10000);
		modemctl = serial_get_info();
		if ((modemctl & TIOCM_DSR) != TIOCM_DSR) {
#ifdef DEBUG
			printf("bt_mode_at ok!\n");
#endif
			return 1;
		}
	}
	return 0;
}

/* 1:N명령어 사용을위한 모드로 전환 */
static int bt_mode_cmdon()
{
	int modemctl;

	while (1) {
		modemctl = serial_get_info();
		if ((modemctl & TIOCM_RTS) != TIOCM_RTS) {
#ifdef DEBUG
			printf("bt_mode_cmd on!\n");
#endif
			return 1;
		}
		_clear_rts();
		usleep(20000);
	}
	return 0;
}

/* 1:N명령어 모드 off */
static int bt_mode_cmdoff()
{
	int modemctl;

	while (1) {
		modemctl = serial_get_info();
		if ((modemctl & TIOCM_RTS) == TIOCM_RTS) {
#ifdef DEBUG
			printf("bt_mode_cmd off!\n");
#endif
			return 1;
		}
		_set_rts();
		usleep(20000);
	}
	return 0;
}

/* 블루투스 펌웨어에 at명령을 입력한다 */
void bt_write_at(char *src, int count)
{
	char cr = 0xd;
//	char *buff = (char *)calloc(PACKET_SIZE, sizeof(char));
	char buff[16] = {0, };

//	serial_set(0, count+1);
	strncpy(buff, src, count);
	buff[count] = cr;

	write(serial_fd, buff, count+1);
//	serial_set(0, PACKET_SIZE);
	usleep(10000);

//	free(buff);
/*	write(serial_fd, src, count);
	write(serial_fd, &cr, 1);*/
}

/* at명령의 결과로 리턴된 문자열을 파싱하여 정보를 얻는다 */
/*static void bt_read_at(char *src)
{
	int i;
	int count = 0, check = 0;
	char buf[PACKET_SIZE] = {0, };
	while (count != -1) {
		read(serial_fd, buf, PACKET_SIZE);
		for (i=0; i<PACKET_SIZE; i++, count++) {
			if (buf[i] == 0x0a) {
				if (check++ == 1) {
					src[count-1] = '\0';
					count = -1;
					break;
				}
				count-=2;
				continue;
			}
			src[count] = buf[i];
		}
	}
}*/


void bt_read_at(char *src)
{
	int i = 0;
	int count = 0;
	int crlf = 0;
	char buf[PACKET_SIZE];
	char *target;

	while (1) {
		read(serial_fd, buf, PACKET_SIZE);
		target = buf;
		for (i=0; i<PACKET_SIZE-1; i++) {
			if (buf[i] == 0xd && buf[i+1] == 0xa) {
				if (crlf++) {
					buf[i] = '\0';
					strncpy((char *)src+count, target, i+1);
					return;
				}
				else {
					target = (char *)buf + 2;
					i++;
				}
			}
		}
		strncpy((char *)src+count, target, PACKET_SIZE);
		count+=PACKET_SIZE;
	}
}

/* 블루투스 소프트 리셋 */
void bt_atz()
{
	char buf[30];

	bt_mode_at();

	bt_write_at("atz", 3);
	bt_read_at(buf);
//	bt_mode_data();

	printf("atz: %s\n", buf);
}

/* 블루투스 정상동작을 확인한다 */
void bt_atcheck()
{
	char buf[5];

	bt_mode_at();

	bt_write_at("at", 2);
//	bt_mode_data();
	bt_read_at(buf);

	printf("at: %s\n", buf);
}

/* 블루투스의 상태정보를 출력한다 */
int bt_btinfo()
{
	char buf[50];
//	printf("btinfo 진입\n");

	bt_mode_at();

	bt_write_at("at+btinfo?", 10);
//	bt_mode_data();
//	printf("btinfo 라이토\n");
	bt_read_at(buf);
//	printf("btinfo 리도\n");

	printf("at+btinfo?: %s\n", buf);

	if (!strcmp(buf, "STANDBY"))
		return 1;
	else if (!strcmp(buf, "PENDING"))
		return 2;
	else if (!strcmp(buf, "CONNECT"))
		return 3;
	return 0;
}

/* 모든 마스터들의 슬레이브에 커넥션 여부를 체크한다 */ 
int bt_concheck()
{
	int modemctl;

	modemctl = serial_get_info();
	if ((modemctl & TIOCM_CAR) == TIOCM_CAR) {
#ifdef DEBUG
		printf("all masters connect complete!\n");
#endif
		return 1;
	}
	return 0;
}

/* 슬레이브에 접속한 마스터들의 주소정보를 알아내고 테이블에 넣어
   고유 장치 번호와 일치시킨다 */
void bt_btinfodev(struct blue_dev *bd, int count)
{
	char buf[20];
	char cmd[15];
	unsigned int addr;
	int i, j;


	for (i=1; i<=count; i++) {
		sprintf(cmd, "at+btinfodev?%d", i);
		bt_mode_at();
		bt_mode_cmdon();
		sleep(1);

		bt_write_at(cmd, 14);

		bt_mode_cmdoff();
		bt_read_at(buf);
//		bt_mode_data();

		printf("at+btinfodev?%d: %s\n", i, buf);

		for (j=0; j<6; j++)
			buf[j] = buf[j+6];
		buf[j] = '\0';
//		printf("buf: %s\n", buf);

		sscanf(buf, "%x", &addr);
//		printf("addr: %x\n", addr);

		for (j=0; j<5; j++) {
			if (addr == bd[j].dev_id) {
//				printf("i: %d\n", i);
				bd[j].ato_num = i;
				break;
			}
		}
//		serial_flush();
	}
#ifdef DEBUG
	for (i=0; i<5; i++)
		printf("id: %d ato_num: %d dev_id: %x\n", i+1, bd[i].ato_num, bd[i].dev_id);
#endif
}


/* 1:N통신시 슬레이브에서 어떤 마스터로 전송해야할지를 선택한다 */
static int bt_ato(unsigned int addr)
{
	static unsigned int saveaddr;
	int modemctl;
	char cmd[20];
	
	if (saveaddr == addr) {
		bt_mode_data();
		return 1;
	}
	saveaddr = addr;

	sprintf(cmd, "ato%d", addr);
	cmd[4] = '\0';
//	cmd[4] = 0xd;
//	printf("cmd: %c %c %c %c\n", cmd[0], cmd[1], cmd[2], cmd[3]);

/*	modemctl = serial_get_info();
	if ((modemctl & TIOCM_DTR) == TIOCM_DTR)*/

//		bt_mode_data();
//		usleep(50000);
		bt_mode_at();
//		sleep(1);
		bt_write_at(cmd, 4);
		usleep(20000);
//		bt_mode_data();

	while (1) {
		modemctl = serial_get_info();
		if ((modemctl & TIOCM_DSR) == TIOCM_DSR) {
			printf("안들어오나?\n");
			bt_mode_data();
#ifdef DEBUG
			printf("master %d select ok!\n", addr);
#endif
			return 1;
		}
		printf("젭라\n");
		usleep(10000);
	}
	return 0;
}

/* 슬레이브에서 검색대기 상태로 들어가게되고 마스터들은 슬레이브를 찾아서 접속한다 */
void bt_scan()
{
	char buf[10];

	bt_mode_at();
	bt_write_at("at+btscan", 9);
	bt_read_at(buf);
//	bt_mode_data();

	printf("at+btscan: %s\n", buf);

	serial_flush();
}

/* 접속한 모든 마스터들에게 패킷을 전송한다 */
void bt_broad(struct blue_dev *masters, int count, struct blue_packet *msg, void *data, size_t byte)
{
	int i;

	for (i=0; i<count; i++) {
		bt_ato(masters[i].ato_num);
		bt_send(msg, data, byte);
#ifdef DEBUG
/*		printf("id:%x write %d %d %d %x\n", masters[i].dev_id, msg->header.src,
		msg->header.dest, msg->ctl, msg->data);*/
#endif
	}
}

/*void bt_send(struct blue_packet *btp, void *data, size_t byte)
{
	static char prevaddr = -1;
	int count = 0;

	if (prevaddr == btp->header.dest)
		usleep(40000);
	prevaddr = btp->header.dest;

	while (count < byte) {
		if (byte <= 12)
			memcpy(&btp->data[0], (char *)data+count, byte);
		else if (byte-count >= 12)
			memcpy(&btp->data[0], (char *)data+count, 12);
		else
			memcpy(&btp->data[0], (char *)data+count, byte-count);
		count += 12;
		write(serial_fd, btp, PACKET_SIZE);
		printf("send: %x %x %x\n", btp->data[0], btp->data[1], btp->data[2]);
	}
	}*/

/*void bt_send(struct blue_packet *btp, void *data, size_t byte)
{
	static char prevaddr = -1;
	int count = 0;

	if (prevaddr == btp->header.dest)
		usleep(40000);
	prevaddr = btp->header.dest;

	while (count < byte) {
		if (byte <= 12)
			memcpy(&btp->data[0], (char *)data+count, byte);
		else if (byte-count >= 12)
			memcpy(&btp->data[0], (char *)data+count, 12);
		else
			memcpy(&btp->data[0], (char *)data+count, byte-count);
		count += 12;
		write(serial_fd, btp, PACKET_SIZE);
		printf("send: %x %x %x\n", btp->data[0], btp->data[1], btp->data[2]);
	}
	}*/

 /* 패킷을 전송한다 */
void bt_send(struct blue_packet *btp, void *data, size_t byte)
{
	static char prevaddr = -1;
	int count = 0;

	if (prevaddr == btp->header.dest)
		usleep(40000);
	prevaddr = btp->header.dest;

	while (count < byte) {
		if (byte-count >= 12)
			memcpy(&btp->data[0], (char *)data+count, 12);
		else
			memcpy(&btp->data[0], (char *)data+count, byte-count);
		count += 12;
		write(serial_fd, btp, PACKET_SIZE);
		printf("send: %x %x %x\n", btp->data[0], btp->data[1], btp->data[2]);
	}
}

/*void bt_send(struct blue_packet *btp, void *data, size_t byte)
{
	static char prevaddr = -1;
	int count = 0;

	if (prevaddr == btp->header.dest)
		usleep(40000);
	prevaddr = btp->header.dest;

	while (count < byte) {
		if (byte <= 12)
			memcpy(&btp->data[0], (char *)data+count, byte);
		else if (byte-count >= 12)
			memcpy(&btp->data[0], (char *)data+count, 12);
		else
			memcpy(&btp->data[0], (char *)data+count, byte-count);
		count += 12;
		write(serial_fd, btp, PACKET_SIZE);
		printf("send: %x %x %x\n", btp->data[0], btp->data[1], btp->data[2]);
	}
	}*/

/*
void bt_send(struct blue_packet *btp, void *data)
{
	struct blue_data d;
	int count;
	int ret;

	if (btp->header.size <= 4) {
		memcpy(&btp->data, data, 4);
		printf("send: %x %x %x\n", btp->header.src, btp->header.dest, btp->data);
		write(serial_fd, btp, PACKET_SIZE);
	} else if (btp->header.size > 4) {
		memcpy(&btp->data, data, 4);
		ret = write(serial_fd, btp, PACKET_SIZE);
		printf("ret: %dbyte\n", ret);
		count = 4;
		sleep(2);
		while (count < btp->header.size) {
			memcpy(&d, (char *)data+count, PACKET_SIZE);
			ret = write(serial_fd, &d, PACKET_SIZE);
			printf("ret: %dbyte\n", ret);
			count += PACKET_SIZE;
		}
	}

}
*/

 /* 모든 마스터들에게 패킷을 보내기위한 랩핑함수 */
void 
bt_send_broad(struct blue_packet *btp, struct blue_dev *masters, int count,
 void *data, size_t byte)
{
	bt_broad(masters, count, btp, data, byte);
}

/* 마스터를 지정하여 패킷을 보낸다 */
void bt_send_target(struct blue_packet *msg, struct blue_dev *masters, void *data, size_t byte)
{
	bt_ato(masters[msg->header.dest-1].ato_num);
	printf("ato번호: %d\n", masters[msg->header.dest-1].ato_num);
	bt_send(msg, data, byte);

#ifdef DEBUG
	printf("id:%x -> %x write %x %x %x %x\n", masters[msg->header.src-1].dev_id,
	       masters[msg->header.dest-1].dev_id, msg->header.src, msg->header.dest,
	       msg->ctl, msg->data[0]);
#endif
}

/* 소스장치번호 데스티네이션장치번호 컨트롤등의 헤더를 넣은 패킷을 만든다 */
void
bt_make_packet(struct blue_packet *btp, unsigned char src,
	       unsigned char dest, unsigned char ctl)
{
	btp->header.src = src;
	btp->header.dest = dest;
	btp->ctl = ctl;
}

/* 다른장치로부터 전송된 패킷을 받는다 */
void bt_recv(struct blue_packet *btp, void *dest, size_t byte)
{
	int count = 0;

 	while (count < byte) {
		read(serial_fd, btp, PACKET_SIZE);
		printf("recv: %x %x %x\n", btp->data[0], btp->data[1], btp->data[2]);
		if (byte-count >= 12)
			memcpy((char *)dest+count, &btp->data[0], 12);
		else
			memcpy((char *)dest+count, &btp->data[0], byte-count);
		count += 12;
	}	
}
/*
void bt_recv(struct blue_packet *btp, void *dest, size_t byte)
{
	struct blue_data d;
	int count;
	void *temp;
	int ret;

	ret = read(serial_fd, btp, PACKET_SIZE);
	printf("ret: %d\n", ret);
	printf("size: %d\n", btp->header.size);
	printf("read: %d %d %d %d\n", btp->header.src, btp->header.dest, btp->ctl, btp->data);
	temp = malloc(btp->header.size + 16);
	printf("malloc\n");
	memcpy(dest, &btp->data, 4);
	printf("memcpy\n");
	count = 4;

	while (count < btp->header.size || count < byte) {
		if (btp->header.size > 4) {
			read(serial_fd, &d, PACKET_SIZE);
			memcpy((char *)temp+count, &d, PACKET_SIZE);
			count += PACKET_SIZE;
		}
	}
	if (btp->header.size > 4) {
		memcpy((char *)dest+4, temp, btp->header.size-4);
		printf("memcpy2\n");
	}
	free(temp);
	printf("free\n");
}
*/













