#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/types.h>
#include <linux/serial.h>    
#include <termios.h>
#include <sys/signal.h>
#include <sys/types.h>

#include "blue.h"

/* /dev/ttySn 디스크립터를 저장할 전역변수 */
static int serial_fd;
int origctl;

/* 시리얼포트 초기화 함수 */
int serial_init(unsigned int baud)
{
	struct termios term;

	serial_fd = open("/dev/ttyS0", O_RDWR | O_NOCTTY);
	if (serial_fd < 0) {
		perror("/dev/ttyS");
		exit(1);
	}

/* termios 구조체는 생성시 모든비트가 1로 초기화됨으로 0으로 셋팅 */
	memset(&term, 0, sizeof(struct termios));

/* 보드레잇 설정및 8비트통신 설정 */
	term.c_cflag = baud | CS8 | CLOCAL | CREAD;
/* 페리티비트를 무시한다 */
	term.c_iflag = IGNPAR;
/* 뉴라인 케리지리턴 상호변환 */
	term.c_oflag = INLCR;
//	term.c_oflag = 0;
/* 아무것도 셋팅하지 않았음으로 non-canonical mode이다 */
	term.c_lflag = 0;

/* non-canonical mode시 셋팅 VTIME에 설정된시간 안에 VMIN만큼 사이즈의
   데이터가 들어오지 않으면 리턴되고 시간관계없이 VMIN에 명시된 사이즈의 패킷이
   들어오면 리턴된다 */
	term.c_cc[VTIME] = 3;
	term.c_cc[VMIN] = PACKET_SIZE;

/* 시리얼 버퍼 플러쉬 */
	tcflush(serial_fd, TCIFLUSH);
/* 시리얼 설정을 바로 적용시킨다 */
	tcsetattr(serial_fd, TCSANOW, &term);

	origctl = serial_get_info();

	return serial_fd;
}

void serial_set(int vtime, int vmin)
{
	struct termios term;

	/* termios 구조체는 생성시 모든비트가 1로 초기화됨으로 0으로 셋팅 */
	memset(&term, 0, sizeof(struct termios));

/* 보드레잇 설정및 8비트통신 설정 */
	term.c_cflag = B38400 | CS8 | CLOCAL | CREAD;
/* 페리티비트를 무시한다 */
	term.c_iflag = IGNPAR;
/* 뉴라인 케리지리턴 상호변환 */
	term.c_oflag = INLCR;
//	term.c_oflag = 0;
/* 아무것도 셋팅하지 않았음으로 non-canonical mode이다 */
	term.c_lflag = 0;

/* non-canonical mode시 셋팅 VTIME에 설정된시간 안에 VMIN만큼 사이즈의
   데이터가 들어오지 않으면 리턴되고 시간관계없이 VMIN에 명시된 사이즈의 패킷이
   들어오면 리턴된다 */
	term.c_cc[VTIME] = vtime;
	term.c_cc[VMIN] = PACKET_SIZE;

/* 시리얼 버퍼 플러쉬 */
	tcflush(serial_fd, TCIFLUSH);
/* 시리얼 설정을 바로 적용시킨다 */
	tcsetattr(serial_fd, TCSANOW, &term);
}



/* 시리얼 플러쉬 랩핑 함수 */
void serial_flush()
{
	tcflush(serial_fd, TCIFLUSH);
}

/* 시리얼 종료 함수 */
void serial_exit()
{
	close(serial_fd);
}

/* 하드웨어 플로우컨트롤 라인을 셋 시킨다 */
static void __serial_set_attr(int modemctl)
{
	ioctl(serial_fd, TIOCMBIS, &modemctl);
	usleep(10000);
}

/* 하드웨어 플로우컨트롤 라인을 클리어 시킨다 */
static void __serial_clear_attr(int modemctl)
{
	ioctl(serial_fd, TIOCMBIC, &modemctl);
	usleep(10000);
}

/* DTR 레지스터 셋 */
void _set_dtr()
{
	__serial_set_attr(TIOCM_DTR);
}

/* DTR 레지스터 클리어 */
void _clear_dtr()
{
	__serial_clear_attr(TIOCM_DTR);
}

/* RTS 레지스터 셋 */
void _set_rts()
{
	__serial_set_attr(TIOCM_RTS);
}

/* RTS 레지스터 클리어 */
void _clear_rts()
{
	__serial_clear_attr(TIOCM_RTS);
}

void _set_default()
{
	ioctl(serial_fd, TIOCMSET, &origctl);
	usleep(10000);
}

/* 시리얼포트 정보를 얻어온다 */
int serial_get_info()
{
	int modemctl;

	ioctl(serial_fd, TIOCMGET, &modemctl);
	usleep(10000);

	return modemctl;
}

/* 시리얼포트 정보를 출력한다 */
void serial_print_info(int modemctl)
{
	if ((modemctl & TIOCM_RTS) == TIOCM_RTS)
		printf("TIOCM_RTS(rs232 RTS 1, ttl CTS[Message Control] 0)\n");
	else
		printf("TIOCM_RTS(rs232 RTS 0, ttl CTS[Message Control] 1)\n");
	if ((modemctl & TIOCM_DTR) == TIOCM_DTR)
		printf("TIOCM_DTR(rs232 DTR 1, ttl DSR[Stream Control] 0)\n");
	else
		printf("TIOCM_DTR(rs232 DTR 0, ttl DSR[Stream Control 1)\n");
//	if ((modemctl & TIOCM_OUT1) == TIOCM_OUT1) printf("TIOCM_OUT1\n");
//	if ((modemctl & TIOCM_OUT2) == TIOCM_OUT2) printf("TIOCM_OUT2\n");
	if ((modemctl & TIOCM_CAR) == TIOCM_CAR)
		printf("TIOCM_CAR(rs232 CAR 1, ttl DCD[Connect Check] 0)\n");
	else
		printf("TIOCM_CAR(rs232 CAR 0, ttl DCD[Connect Check] 1)\n");
	if ((modemctl & TIOCM_RNG) == TIOCM_RNG)
		printf("TIOCM_RNG(rs232 RNG 1)\n");
	else
		printf("TIOCM_RNG(rs232 RNG 0)\n");
	if ((modemctl & TIOCM_DSR) == TIOCM_DSR)
		printf("TIOCM_DSR(rs232 DSR 1, ttl DTR[Stream Status] 0)\n");
	else
		printf("TIOCM_DSR(rs232 DSR 0, ttl DTR[Stream Status] 1)\n");

	if ((modemctl & TIOCM_CTS) == TIOCM_CTS)
		printf("TIOCM_CTS(rs232 CTS 1, ttl RTS[Message Status] 0)\n");
	else
		printf("TIOCM_CTS(rs232 CTS 0, ttl RTS[Message Status] 1)\n");
}

