#include "common.h"
#include "adc.h"
#include "blue.h"
#include "menu.h"

#define DEBUG

/******************************************************************************************************
 * 
 * < main 함수 >
 *
 * 1. 프레임버퍼, 터치스크린, 사운드 장치 드라이버 오픈
 * 2. 프로그램 초기화, 프레임버퍼 초기화, 블루투스 초기화
 * 3. 터치 스크린 인터페이스를 위한 ts_thread 생성
 * 4. 가속도 감지를 위한 acc_thread 생성
 * 5. 바람 감지를 위한 wind_thread 생성
 * 6. 메뉴에서 선택된 프로그램 실행
 * 
 *******************************************************************************************************/
int main(int argc, char **argv)
{
	pid_t pid;
	int stat_val;


	fb_fd=open(FB_DEV, O_RDWR);
	if( fb_fd < 0 )
    	{
		printf("%s open failed\n", FB_DEV);
		exit(-1);
    	}

    	ts_fd=open(TS_DEV, O_RDWR);
    	if( ts_fd < 0 )
    	{
		printf("%s open failed\n", TS_DEV);
		exit(-1);
    	}
    	
	sound_fd=open(SOUND_DEV, O_RDWR);
    	if( sound_fd < 0 )
    	{
		printf("%s open failed\n", SOUND_DEV);
		exit(-1);
    	}

   	pfbdata=(unsigned short *) mmap(0, FB_SIZE*2, PROT_READ | PROT_WRITE, MAP_SHARED, fb_fd, 0);

	signal(SIGINT, cleanup);

	init_fb();

	init_sudo_load_image();

	display_image_to_fb(sudo_logo, SUDO_LOGO_IMAGE_X_SIZE, SUDO_LOGO_IMAGE_Y_SIZE,
			SUDO_LOGO_IMAGE_X_OFFSET, SUDO_LOGO_IMAGE_Y_OFFSET);

	pthread_create(&sudo_load_thread, NULL, sudo_load_thread_func, NULL);

	open_menu_image();

	open_menu_wav();

	init_menu_instance();

	load_complete_flag=1;

	pthread_join(sudo_load_thread, &thread_result);

	init_fb();

	display_menu_image();

	pthread_mutex_init(&mutex, NULL);

	if( pthread_create(&ts_thread, NULL, ts_thread_func, NULL) != 0 ) 
	{
		printf("Create ts_tread failed\n");
		exit(-1);
	}

	if( pthread_create(&acc_thread, NULL, acc_thread_func, NULL) != 0 )
	{
		printf("Create acc_thread  failed\n");
		exit(-1);
	}

	if( pthread_create(&blue_thread, NULL, blue_thread_func, NULL) != 0 )
	{
		printf("Create blue_thread failed\n");
		exit(-1);
	}

	while(1)
	{
		if(exec_app)
		{
			pause_menu();

			pid=fork();

			if( pid == 0)
			{
				if(current->sub_menu_cnt)
				{
					printf("%s execute\n", current->sub[sub_index]->app_name);
					execl(current->sub[sub_index]->app_path, 
							current->sub[sub_index]->app_name, 0);
				}else
				{
					printf("%s execute\n", current->app_name);
					execl(current->app_path, current->app_name, 0);
				}
			}

			wait(&stat_val);

			pthread_mutex_lock(&mutex);
			exec_app=0;
			pthread_mutex_unlock(&mutex);

			restart_menu();
		}

		if(t_puzzle_cli_exec)
		{
			pause_menu();

			pid=fork();

			if( pid == 0 )
			{
				printf("t_puzzle_cli_app execute\n");
				execl(EDU_PUZZLE_CLI_APP_PATH, EDU_PUZZLE_CLI_APP, 0);
			}

			wait(&stat_val);

			pthread_mutex_lock(&mutex);
			t_puzzle_cli_exec=0;
			pthread_mutex_unlock(&mutex);


			restart_menu();
		}
		
		if(ctl_pocketmon_exec)
		{
			pause_menu();

			pid=fork();

			if( pid == 0 )
			{
				printf("ctl_pocketmon_gmae execute\n");
				execl(GAME_POCKETMON_CTL_APP_PATH, GAME_POCKETMON_CTL_APP, 0);
			}

			wait(&stat_val);

			pthread_mutex_lock(&mutex);
			ctl_pocketmon_exec=0;
			pthread_mutex_unlock(&mutex);

			restart_menu();
		}

		if(power_off)
			cleanup();
	}
	
    	cleanup();
}

/********************************************************************************************
 *
 *  < void init_sudo_load_image(void) >
 *
 *  - 메뉴 로딩 이미지를 메모리에 로드
 *
 ********************************************************************************************/
void init_sudo_load_image(void)
{/*{{{*/
	int fd;
	
	fd=open_image(SUDO_LOGO_IMAGE_PATH);
	copy_image_to_buff(fd, sudo_logo, SUDO_LOGO_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(SUDO_LOAD_IMAGE1_PATH);
	copy_image_to_buff(fd, sudo_loading[0], SUDO_LOAD_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(SUDO_LOAD_IMAGE2_PATH);
	copy_image_to_buff(fd, sudo_loading[1], SUDO_LOAD_IMAGE_SIZE);
	close(fd);

	fd=open_image(SUDO_LOAD_IMAGE3_PATH);
	copy_image_to_buff(fd, sudo_loading[2], SUDO_LOAD_IMAGE_SIZE);
	close(fd);

	fd=open_image(SUDO_LOAD_IMAGE4_PATH);
	copy_image_to_buff(fd, sudo_loading[3], SUDO_LOAD_IMAGE_SIZE);
	close(fd);

	fd=open_image(SUDO_LOAD_IMAGE5_PATH);
	copy_image_to_buff(fd, sudo_loading[4], SUDO_LOAD_IMAGE_SIZE);
	close(fd);

	fd=open_image(SUDO_LOAD_IMAGE6_PATH);
	copy_image_to_buff(fd, sudo_loading[5], SUDO_LOAD_IMAGE_SIZE);
	close(fd);

	fd=open_image(SUDO_LOAD_IMAGE7_PATH);
	copy_image_to_buff(fd, sudo_loading[6], SUDO_LOAD_IMAGE_SIZE);
	close(fd);

	fd=open_image(SUDO_LOAD_IMAGE8_PATH);
	copy_image_to_buff(fd, sudo_loading[7], SUDO_LOAD_IMAGE_SIZE);
	close(fd);

	fd=open_image(SUDO_LOAD_IMAGE9_PATH);
	copy_image_to_buff(fd, sudo_loading[8], SUDO_LOAD_IMAGE_SIZE);
	close(fd);

	fd=open_image(SUDO_LOAD_IMAGE10_PATH);
	copy_image_to_buff(fd, sudo_loading[9], SUDO_LOAD_IMAGE_SIZE);
	close(fd);

	fd=open_image(SUDO_LOAD_IMAGE11_PATH);
	copy_image_to_buff(fd, sudo_loading[10], SUDO_LOAD_IMAGE_SIZE);
	close(fd);
}/*}}}*/

/***********************************************************************************
 *
 * < void open_menu_image(void) >
 *
 * - 메뉴 프로그램에 필요한 이미지 파일을 메모리로 로드한다
 *
 ***********************************************************************************/
void open_menu_image(void)
{/*{{{*/
	int fd;
	int ret;

	fd=open_image(MENU_LEFT_ARROW_PATH);
	copy_image_to_buff(fd, left_arrow, ARROW_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_LEFT_ARROW_H_PATH);
	copy_image_to_buff(fd, left_arrow_h, ARROW_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_RIGHT_ARROW_PATH);
	copy_image_to_buff(fd, right_arrow, ARROW_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_RIGHT_ARROW_H_PATH);
	copy_image_to_buff(fd, right_arrow_h, ARROW_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_UP_ARROW_PATH);
	copy_image_to_buff(fd, up_arrow, ARROW_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_UP_ARROW_H_PATH);
	copy_image_to_buff(fd, up_arrow_h, ARROW_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_DOWN_ARROW_PATH);
	copy_image_to_buff(fd, down_arrow, ARROW_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_DOWN_ARROW_H_PATH);
	copy_image_to_buff(fd, down_arrow_h, ARROW_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_ARROW_HIDE_PATH);
	copy_image_to_buff(fd, arrow_hide, ARROW_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(POWER_IMAGE_PATH);
	copy_image_to_buff(fd, power, POWER_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(POWER_H_IMAGE_PATH);
	copy_image_to_buff(fd, power_h, POWER_IMAGE_SIZE);
	close(fd);

	fd=open_image(PLAY_IMAGE_PATH);
	copy_image_to_buff(fd, play, PLAY_IMAGE_SIZE);
	close(fd);

	fd=open_image(PLAY_H_IMAGE_PATH);
	copy_image_to_buff(fd, play_h, PLAY_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(INFO_IMAGE_PATH);
	copy_image_to_buff(fd, info, INFO_IMAGE_SIZE);
	close(fd);
		
	fd=open_image(INFO_H_IMAGE_PATH);
	copy_image_to_buff(fd, info_h, INFO_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(HOLD_TOUCH_IMAGE_PATH);
	copy_image_to_buff(fd, touch, HOLD_IMAGE_SIZE);
	close(fd);

	fd=open_image(HOLD_ACC_IMAGE_PATH);
	copy_image_to_buff(fd, acc, HOLD_IMAGE_SIZE);
	close(fd);

	fd=open_image(HOLD_ALL_IMAGE_PATH);
	copy_image_to_buff(fd, all, HOLD_IMAGE_SIZE);
	close(fd);

	fd=open_image(NEXT_IMAGE_PATH);
	copy_image_to_buff(fd, next, NEXT_IMAGE_SIZE);
	close(fd);

	fd=open_image(NEXT_H_IMAGE_PATH);
	copy_image_to_buff(fd, next_h, NEXT_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_IMAGE_PATH);
	copy_image_to_buff(fd, stop, STOP_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_H_IMAGE_PATH);
	copy_image_to_buff(fd, stop_h, STOP_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_TILT_IMAGE1_PATH);
	copy_image_to_buff(fd, tilt.image1, MENU_IMAGE1_SIZE);
	close(fd);
	/*
	fd=open_image(MENU_TILT_IMAGE2_PATH);
	copy_image_to_buff(fd, tilt.image2, MENU_IMAGE2_SIZE);
	close(fd);
	*/
	fd=open_image(MENU_TILT_IMAGE3_PATH);
	copy_image_to_buff(fd, tilt.image3, MENU_IMAGE3_SIZE);
	close(fd);
	
	fd=open_image(MENU_EDU_IMAGE1_PATH);
	copy_image_to_buff(fd, edu.image1, MENU_IMAGE1_SIZE);
	close(fd);
	/*
	fd=open_image(MENU_EDU_IMAGE2_PATH);
	ret=copy_image_to_buff(fd, edu.image2, MENU_IMAGE2_SIZE);
	printf("%d Bytes copy to buff\n", ret*2);
	close(fd);
	*/
	fd=open_image(MENU_EDU_IMAGE3_PATH);
	copy_image_to_buff(fd, edu.image3, MENU_IMAGE3_SIZE);
	close(fd);
	
	fd=open_image(MENU_DEVICE_IMAGE1_PATH);
	copy_image_to_buff(fd, device.image1, MENU_IMAGE1_SIZE);
	close(fd);
	/*
	fd=open_image(MENU_DEVICE_IMAGE2_PATH);
	ret=copy_image_to_buff(fd, device.image2, MENU_IMAGE2_SIZE);
	//printf("%d Bytes copy to buff\n", ret*2);
	close(fd);
	*/
	fd=open_image(MENU_DEVICE_IMAGE3_PATH);
	copy_image_to_buff(fd, device.image3, MENU_IMAGE3_SIZE);
	close(fd);

	fd=open_image(MENU_PICA_IMAGE1_PATH);
	copy_image_to_buff(fd, pica.image1, MENU_IMAGE1_SIZE);
	close(fd);
	/*
	fd=open_image(MENU_PICA_IMAGE2_PATH);
	copy_image_to_buff(fd, pica.image2, MENU_IMAGE2_SIZE);
	close(fd);
	*/
	fd=open_image(MENU_PICA_IMAGE3_PATH);
	copy_image_to_buff(fd, pica.image3, MENU_IMAGE3_SIZE);
	close(fd);
	
	fd=open_image(SUB_MENU_SHOCK_IMAGE_PATH);
	copy_image_to_buff(fd, shock.image1, MENU_IMAGE1_SIZE);
	close(fd);
	
	fd=open_image(SUB_MENU_SHAKE_IMAGE_PATH);
	copy_image_to_buff(fd, shake.image1, MENU_IMAGE1_SIZE);
	close(fd);
	
	fd=open_image(SUB_MENU_EWSN_IMAGE_PATH);
	copy_image_to_buff(fd, ewsn.image1, MENU_IMAGE1_SIZE);
	close(fd);
	
	fd=open_image(SUB_MENU_DROP_IMAGE_PATH);
	copy_image_to_buff(fd, drop.image1, MENU_IMAGE1_SIZE);
	close(fd);
	
	fd=open_image(SUB_MENU_WIND_IMAGE_PATH);
	copy_image_to_buff(fd, wind.image1, MENU_IMAGE1_SIZE);
	close(fd);
	
	fd=open_image(SUB_MENU_NEIGHBOR_IMAGE_PATH);
	copy_image_to_buff(fd, neighbor.image1, MENU_IMAGE1_SIZE);
	close(fd);

	fd=open_image(SUB_MENU_FIGURES_IMAGE_PATH);
	copy_image_to_buff(fd, figures.image1, MENU_IMAGE1_SIZE);
	close(fd);

	fd=open_image(SUB_MENU_T_PUZZLE_IMAGE_PATH);
	copy_image_to_buff(fd, t_puzzle.image1, MENU_IMAGE1_SIZE);
	close(fd);

	fd=open_image(MENU_MANUAL_IMAGE1_PATH);
	copy_image_to_buff(fd, menu_manual[0], MENU_MANUAL_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_MANUAL_IMAGE2_PATH);
	copy_image_to_buff(fd, menu_manual[1], MENU_MANUAL_IMAGE_SIZE);
	close(fd);

	fd=open_image(MENU_MANUAL_IMAGE3_PATH);
	copy_image_to_buff(fd, menu_manual[2], MENU_MANUAL_IMAGE_SIZE);
	close(fd);
}/*}}}*/

/***********************************************************************************
 *
 * < void open_menu_wav(void) >
 *
 * - 메뉴 프로그램에 필요한 사운드 파일을 메모리로 로드한다
 *
 ***********************************************************************************/
void open_menu_wav(void)
{/*{{{*/
	int fd;

	fd=open_wav(POWER_WAV_PATH);
	copy_wav_to_buff(fd, power_wav, POWER_WAV_SIZE);
	close(fd);
	
	fd=open_wav(CLICK_WAV_PATH);
	copy_wav_to_buff(fd, click_wav, CLICK_WAV_SIZE);
	close(fd);
	
	fd=open_wav(APP_START_WAV_PATH);
	copy_wav_to_buff(fd, app_start_wav, APP_START_WAV_SIZE);
	close(fd);

	fd=open_wav(INFO_WAV_PATH);
	copy_wav_to_buff(fd, info_wav, INFO_WAV_SIZE);
	close(fd);

	fd=open_wav(HOLD_WAV_PATH);
	copy_wav_to_buff(fd, hold_wav, HOLD_WAV_SIZE);
	close(fd);
}/*}}}*/

/*********************************************************************************************
 *
 *  < void display_menu_image(void) >
 *
 *  - 메뉴 프로그램과 관련된 초기 이미지 파일을 화면에 보여준다
 *
 *********************************************************************************************/
void display_menu_image(void)
{/*{{{*/
	display_image_to_fb(left_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			LEFT_ARROW_X_OFFSET, LEFT_ARROW_Y_OFFSET);
	
	display_image_to_fb(right_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE, 
			RIGHT_ARROW_X_OFFSET, RIGHT_ARROW_Y_OFFSET);

	if(current->sub_menu_cnt)
	{
		display_image_to_fb(up_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE, 
				UP_ARROW_X_OFFSET, UP_ARROW_Y_OFFSET);

		display_image_to_fb(down_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE, 
				DOWN_ARROW_X_OFFSET, DOWN_ARROW_Y_OFFSET);
	}
	
	display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
			POWER_X_OFFSET, POWER_Y_OFFSET);

	display_image_to_fb(play, PLAY_IMAGE_X_SIZE, PLAY_IMAGE_Y_SIZE,
			PLAY_IMAGE_X_OFFSET, PLAY_IMAGE_Y_OFFSET);

	display_image_to_fb(info, INFO_IMAGE_X_SIZE, INFO_IMAGE_Y_SIZE,
			INFO_IMAGE_X_OFFSET, INFO_IMAGE_Y_OFFSET);

	switch(hold_index)
	{
		case ENABLE_ALL:
			
			display_image_to_fb(all, HOLD_IMAGE_X_SIZE, HOLD_IMAGE_Y_SIZE,
					HOLD_IMAGE_X_OFFSET, HOLD_IMAGE_Y_OFFSET);
			break;

		case ENABLE_ACC:

			display_image_to_fb(acc, HOLD_IMAGE_X_SIZE, HOLD_IMAGE_Y_SIZE,
					HOLD_IMAGE_X_OFFSET, HOLD_IMAGE_Y_OFFSET);
			break;

		case ENABLE_TOUCH:

			display_image_to_fb(touch, HOLD_IMAGE_X_SIZE, HOLD_IMAGE_Y_SIZE,
					HOLD_IMAGE_X_OFFSET, HOLD_IMAGE_Y_OFFSET);
			break;
	}
	
	display_image_to_fb(current->image1, MENU_IMAGE1_X_SIZE, MENU_IMAGE1_Y_SIZE, 
			CENTER_X_OFFSET, CENTER_Y_OFFSET);

	display_image_to_fb(left->image3, MENU_IMAGE3_X_SIZE, MENU_IMAGE3_Y_SIZE, 
			LEFT_X_OFFSET, LEFT_Y_OFFSET);

	display_image_to_fb(right->image3, MENU_IMAGE3_X_SIZE, MENU_IMAGE3_Y_SIZE,
			RIGHT_X_OFFSET, RIGHT_Y_OFFSET);
}/*}}}*/

/**********************************************************************************************
 *
 *  < void display_menu_info_image(void) >
 *
 *  - 사용자가 인포 버튼을 눌렀을 경우에 해당 도움말을 보여주고 터치 스크린을 통해
 *    도움말 인터페이스를 컨트롤 한다
 *
 *********************************************************************************************/
void display_menu_info_image(void)
{/*{{{*/
	int ret;
	int step_cnt=0;
	short ts_buf[4];
		
	display_image_to_fb(menu_manual[step_cnt], MENU_MANUAL_IMAGE_X_SIZE, MENU_MANUAL_IMAGE_Y_SIZE,
				MENU_MANUAL_IMAGE_X_OFFSET, MENU_MANUAL_IMAGE_Y_OFFSET);
	
	display_image_to_fb(next, NEXT_IMAGE_X_SIZE, NEXT_IMAGE_Y_SIZE,
			NEXT_IMAGE_X_OFFSET, NEXT_IMAGE_Y_OFFSET);

	step_cnt=1;

	while(1)
	{
		ret=read(ts_fd, ts_buf, sizeof(ts_buf));
		if( ret < 0 )
		{
	    		printf("Read Touch Screen Data Error\n");
			exit(-1);
		}

		ret=check_click_zone(ts_buf[1], ts_buf[2]);

		if( ts_buf[0] && ret == NEXT_STOP_CLICKED )
		{

			if( step_cnt == 3 )
			{
				display_image_to_fb(stop_h, STOP_IMAGE_X_SIZE, STOP_IMAGE_Y_SIZE,
						STOP_IMAGE_X_OFFSET, STOP_IMAGE_Y_OFFSET);
				
				play_wav(click_wav, CLICK_WAV_SIZE);

				break;
			}else
			{
				display_image_to_fb(next_h, NEXT_IMAGE_X_SIZE, NEXT_IMAGE_Y_SIZE,
						NEXT_IMAGE_X_OFFSET, NEXT_IMAGE_Y_OFFSET);
				
				play_wav(click_wav, CLICK_WAV_SIZE);
			}
			
			display_image_to_fb(menu_manual[step_cnt], MENU_MANUAL_IMAGE_X_SIZE, MENU_MANUAL_IMAGE_Y_SIZE,
					MENU_MANUAL_IMAGE_X_OFFSET, MENU_MANUAL_IMAGE_Y_OFFSET);

			if( step_cnt < 2 )
			{
				display_image_to_fb(next, NEXT_IMAGE_X_SIZE, NEXT_IMAGE_Y_SIZE,
						NEXT_IMAGE_X_OFFSET, NEXT_IMAGE_Y_OFFSET);
			}else
			{
				display_image_to_fb(stop, STOP_IMAGE_X_SIZE, STOP_IMAGE_Y_SIZE,
						STOP_IMAGE_X_OFFSET, STOP_IMAGE_Y_OFFSET);
			}

			step_cnt++;
		}
	}
}/*}}}*/

/******************************************************************************************************
 *
 *  < void init_menu_instance(void) >
 *
 *  - 메뉴 프로그램에서 사용될 메뉴 인스턴스들을 초기화해 준다
 *
 *****************************************************************************************************/
void init_menu_instance(void)
{/*{{{*/
	int index;

	tilt.state=POS_CENTER;
	tilt.index=1;
	tilt.sub_menu_cnt=5;
	tilt.app_name=TILT_SHAKE_APP;;
	tilt.app_path=TILT_SHAKE_APP_PATH;
	tilt.sub[0]=&shake;
	shake.app_name=TILT_SHAKE_APP;
	shake.app_path=TILT_SHAKE_APP_PATH;
	tilt.sub[1]=&shock;
	shock.app_name=TILT_SHOCK_APP;
	shock.app_path=TILT_SHOCK_APP_PATH;
	tilt.sub[2]=&ewsn;
	ewsn.app_name=TILT_DIRECTION_APP;
	ewsn.app_path=TILT_DIRECTION_APP_PATH;
	tilt.sub[3]=&drop;
	drop.app_name=TILT_DROP_APP;
	drop.app_path=TILT_DROP_APP_PATH;
	tilt.sub[4]=&neighbor;
	neighbor.app_name=TILT_NEIGHBOR_APP;
	neighbor.app_path=TILT_NEIGHBOR_APP_PATH;
	tilt.sub[5]=&wind;
	wind.app_name=TILT_WIND_APP;
	wind.app_path=TILT_WIND_APP_PATH;

	edu.state=POS_LEFT;
	edu.index=2;
	edu.sub_menu_cnt=1;
	edu.sub[0]=&figures;
	figures.app_name=EDU_FIGURES_APP;
	figures.app_path=EDU_FIGURES_APP_PATH;
	edu.sub[1]=&t_puzzle;
	t_puzzle.app_name=EDU_PUZZLE_SERV_APP;
	t_puzzle.app_path=EDU_PUZZLE_SERV_APP_PATH;
	edu.app_name=EDU_FIGURES_APP;
	edu.app_path=EDU_FIGURES_APP_PATH;

	device.state=POS_RIGHT;
	device.index=3;
	device.sub_menu_cnt=0;
	device.app_name=DEVICE_BULB_APP;
	device.app_path=DEVICE_BULB_APP_PATH;

	pica.state=POS_HIDDEN;
	pica.index=4;
	pica.sub_menu_cnt=0;
	pica.app_name=GAME_POCKETMON_APP;
	pica.app_path=GAME_POCKETMON_APP_PATH;

	current=&tilt;
	left=&edu;
	right=&device;
	hidden=&pica;

	up_down_arrow_key=1;
}/*}}}*/

/*********************************************************************************************************
 *
 *  < int check_click_zone(short x, short, y) >
 *
 *  - 터치 스크린 디바이스로 부터 얻어온 x, y좌표 값을 받아 설정해 놓은 구역에 합당하면
 *    해당 리턴값을 반환한다
 *
 *********************************************************************************************************/
int check_click_zone(short x, short y)
{/*{{{*/
	if( x > 2 && x < 32 )
	{
		if( y > 224 && y < 256 )
			return LEFT_CLICKED;
	}

	if( x > 590 && x < 638 )
	{
		if( y > 224 && y < 256 )
			return RIGHT_CLICKED;
	}

	if( x > 304 && x < 336 )
	{
		if( y > 2 && y < 32)
			return UP_CLICKED;
	}
	
	if( x > 304 && x < 336 )
	{
		if( y > 448 && y < 478 )
			return DOWN_CLICKED;
	}

	if( x > 560 && x < 630 )
	{
		if( y > 394 && y < 464 )
			return PLAY_CLICKED;
	}

	if( x > 560 && x < 630 )
	{
		if( y > 16 && y < 86 )
			return POWER_CLICKED;
	}

	if( x > 16 && x < 86 )
	{
		if( y > 16 && y < 86 )
			return INFO_CLICKED;
	}

	if( x > 16 && x < 86 )
	{
		if( y > 394 && y < 464 )
			return HOLD_CLICKED;
	}

	if( x > 460 && x < 500 )
	{
		if( y > 320 && y < 360 )
			return NEXT_STOP_CLICKED;
	}
}/*}}}*/

/********************************************************************************************************
 *
 *  < void clicked_left(void) >
 *
 *  - 사용자가 윈쪽 화살표를 클릭 혹은 기울여서 선택했을 경우, 화살표를 반짝여주고 메뉴를 업데이트 한다.
 *
 ********************************************************************************************************/
void clicked_left(void)
{/*{{{*/
	display_image_to_fb(left_arrow_h, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			LEFT_ARROW_X_OFFSET, LEFT_ARROW_Y_OFFSET);
	
	update_menu(LEFT_CLICKED);
	
	display_image_to_fb(left_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			LEFT_ARROW_X_OFFSET, LEFT_ARROW_Y_OFFSET);

	play_wav(click_wav, CLICK_WAV_SIZE);
}/*}}}*/

/********************************************************************************************************
 *
 *  < void clicked_right(void) >
 *
 *  - 사용자가 오른쪽 화살표를 클릭 혹은 기울여서 선택했을 경우, 화살표를 반짝여주고 메뉴를 업데이트 한다.
 *
 ********************************************************************************************************/
void clicked_right(void)
{/*{{{*/
        display_image_to_fb(right_arrow_h, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
		       RIGHT_ARROW_X_OFFSET, RIGHT_ARROW_Y_OFFSET);
	
	update_menu(RIGHT_CLICKED);
        
	display_image_to_fb(right_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			RIGHT_ARROW_X_OFFSET, RIGHT_ARROW_Y_OFFSET);

	play_wav(click_wav, CLICK_WAV_SIZE);
}/*}}}*/

/********************************************************************************************************
 *
 *  < void clicked_up(void) >
 *
 *  - 사용자가 위쪽 화살표를 클릭 혹은 기울여서 선택했을 경우, 화살표를 반짝여주고 메뉴를 업데이트 한다.
 *
 ********************************************************************************************************/
void clicked_up(void)
{/*{{{*/
	display_image_to_fb(up_arrow_h, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			UP_ARROW_X_OFFSET, UP_ARROW_Y_OFFSET);
	
	update_menu(UP_CLICKED);
	
	display_image_to_fb(up_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			UP_ARROW_X_OFFSET, UP_ARROW_Y_OFFSET);

	play_wav(click_wav, CLICK_WAV_SIZE);
}/*}}}*/

/********************************************************************************************************
 *
 *  < void clicked_down(void) >
 *
 *  - 사용자가 아래쪽 화살표를 클릭 혹은 기울여서 선택했을 경우, 화살표를 반짝여주고 메뉴를 업데이트 한다.
 *
 ********************************************************************************************************/
void clicked_down(void)
{/*{{{*/
	display_image_to_fb(down_arrow_h, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			DOWN_ARROW_X_OFFSET, DOWN_ARROW_Y_OFFSET);
	
	update_menu(DOWN_CLICKED);
	
	display_image_to_fb(down_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			DOWN_ARROW_X_OFFSET, DOWN_ARROW_Y_OFFSET);

	play_wav(click_wav, CLICK_WAV_SIZE);
}/*}}}*/

/*******************************************************************************************************
 *
 *  < void update_menu(int clicked_direction) >
 *
 *  - 사용자가 터치 혹은 모듈을 기울여 메뉴를 선택하면 current, left, right, hidden에 변경된 메뉴
 *    인스턴스를 연결해 주고 변경된 사항에 대한 이미지를 화면에 보여준다.
 *
 *******************************************************************************************************/
void update_menu(int clicked_direction)
{/*{{{*/
	struct menu_instance *temp1;
	struct menu_instance *temp2;

	switch(clicked_direction)
	{
		case LEFT_CLICKED:

			display_image_to_fb(right->image1, MENU_IMAGE1_X_SIZE, MENU_IMAGE1_Y_SIZE,
					CENTER_X_OFFSET, CENTER_Y_OFFSET);
			display_image_to_fb(current->image3, MENU_IMAGE3_X_SIZE, MENU_IMAGE3_Y_SIZE,
					LEFT_X_OFFSET,LEFT_Y_OFFSET);
			display_image_to_fb(hidden->image3, MENU_IMAGE3_X_SIZE, MENU_IMAGE3_Y_SIZE,
					RIGHT_X_OFFSET, RIGHT_Y_OFFSET);
			temp1=current;
			temp2=hidden;

			current=right;
			hidden=left;
			left=temp1;
			right=temp2;

			if(current->sub_menu_cnt)
			{
				sub_index=current->sub_menu_cnt;
				up_down_arrow_unlock();
			}
			else
				up_down_arrow_lock();

			break;
		
		case RIGHT_CLICKED:

			display_image_to_fb(left->image1, MENU_IMAGE1_X_SIZE, MENU_IMAGE1_Y_SIZE,
					CENTER_X_OFFSET, CENTER_Y_OFFSET);
			display_image_to_fb(current->image3, MENU_IMAGE3_X_SIZE, MENU_IMAGE3_Y_SIZE,
					RIGHT_X_OFFSET, RIGHT_Y_OFFSET);
			display_image_to_fb(hidden->image3, MENU_IMAGE3_X_SIZE, MENU_IMAGE3_Y_SIZE,
					LEFT_X_OFFSET, LEFT_Y_OFFSET);

			temp1=current;
			temp2=hidden;
			
			current=left;
			hidden=right;
			right=temp1;
			left=temp2;

			if(current->sub_menu_cnt)
			{
				sub_index=current->sub_menu_cnt;
				up_down_arrow_unlock();
			}
			else
				up_down_arrow_lock();

			break;
		
		case UP_CLICKED:

			sub_index--;

			if( sub_index < 0 )
				sub_index=current->sub_menu_cnt;
			
			display_image_to_fb(current->sub[sub_index]->image1, MENU_IMAGE1_X_SIZE,
					MENU_IMAGE1_Y_SIZE, CENTER_X_OFFSET, CENTER_Y_OFFSET);

			break;

		case DOWN_CLICKED:

			sub_index++;
			
			if( sub_index > current->sub_menu_cnt )
				sub_index=0;

			display_image_to_fb(current->sub[sub_index]->image1, MENU_IMAGE1_X_SIZE,
					MENU_IMAGE1_Y_SIZE, CENTER_X_OFFSET, CENTER_Y_OFFSET);
			
			break;
	}
}/*}}}*/

/********************************************************************************************************
 *
 * < void *ts_thread_func(void *arg) >
 *
 * - 터치 스크린 인터페이스를 위한 함수
 *	LEFT_CLICKED	: 왼쪽 화살표를 클릭했을시의 동작 실행 
 *	RIGHT_CLICKED	: 오른쪽 화살표를 클릭했을시의 동작 실행
 *	UP_CLICKED	: 위쪽 화살표를 클릭했을시의 동작 실행
 *	DOWN_CLICKED	: 아래쪽 화살표를 클릭했을시의 동작 실행
 *	PLAY_CLICKED	: 실행 버튼을 클릭했을시의 동작 실행
 *	POWER_CLICKED	: 전원 버튼을 클릭했을시의 동작 실행
 *	INFO_CLICKED	: 도움말 버튼을 클릭했을시의 동작 실행
 *	HOLD_CLICKED	: 홀드 버튼을 클릭했을시의 동작 실행
 *		ENABLE_ALL	: 터치스크린 가속도 인터페이스를 둘다 활성화 한다
 *		ENABLE_ACC	: 가속도 인터페이스만 활성화 한다
 *		ENABLE_TOUCH	: 터치스크린 인터페이스만 활성화 한다.
 *
 ********************************************************************************************************/
void *ts_thread_func(void *arg)
{/*{{{*/

	int direction;
	short ts_buf[4];

	int old_acc_enable;
	
	while(1)
	{
		if(read(ts_fd,ts_buf,sizeof(ts_buf)) < 0)
		{
	    		printf("Read Touch Screen Data Error\n");
	    		cleanup();
		}	
#ifdef DEBUG	
		printf("Press[%d] X[%d] Y[%d] MillSec[%d]\n", ts_buf[0], ts_buf[1], ts_buf[2], ts_buf[3]);
#endif
		direction=check_click_zone(ts_buf[1], ts_buf[2]);
		
		if(ts_buf[0])
		{
			switch(direction)
			{
				case LEFT_CLICKED:
					
					if(touch_enable)
						clicked_left();
					break;
				
				case RIGHT_CLICKED:

					if(touch_enable)
						clicked_right();
					break;
				
				case UP_CLICKED:

					if( touch_enable && up_down_arrow_key )
						clicked_up();
					break;
				
				case DOWN_CLICKED:

					if( touch_enable && up_down_arrow_key )
						clicked_down();
					break;
				
				case PLAY_CLICKED:

					display_image_to_fb(play_h, PLAY_IMAGE_X_SIZE, PLAY_IMAGE_Y_SIZE,
							PLAY_IMAGE_X_OFFSET, PLAY_IMAGE_Y_OFFSET);

					play_wav(app_start_wav, APP_START_WAV_SIZE);

					pthread_mutex_lock(&mutex);
					exec_app=1;
					pthread_mutex_unlock(&mutex);

					break;

				case POWER_CLICKED:

					display_image_to_fb(power_h, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
							POWER_X_OFFSET, POWER_Y_OFFSET);
					
					play_wav(power_wav, POWER_WAV_SIZE);

					display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
							POWER_X_OFFSET, POWER_Y_OFFSET);

					pthread_mutex_lock(&mutex);
					power_off=1;
					pthread_mutex_unlock(&mutex);

				case INFO_CLICKED:

					old_acc_enable=acc_enable;

					pthread_mutex_lock(&mutex);
					acc_enable=0;
					pthread_mutex_unlock(&mutex);

					display_image_to_fb(info_h, INFO_IMAGE_X_SIZE, INFO_IMAGE_Y_SIZE,
							INFO_IMAGE_X_OFFSET, INFO_IMAGE_Y_OFFSET);

					play_wav(info_wav, INFO_WAV_SIZE);

					display_menu_info_image();

					init_fb();

					display_menu_image();

					pthread_mutex_lock(&mutex);
					acc_enable=old_acc_enable;
					pthread_mutex_unlock(&mutex);
					
					break;

				case HOLD_CLICKED:

					if( hold_index == 3 )
						hold_index=1;
					else
						hold_index++;

					switch(hold_index)
					{
						case ENABLE_ALL:

							display_image_to_fb(all, HOLD_IMAGE_X_SIZE, HOLD_IMAGE_Y_SIZE,
									HOLD_IMAGE_X_OFFSET, HOLD_IMAGE_Y_OFFSET);

							pthread_mutex_lock(&mutex);
							touch_enable=1;
							acc_enable=1;
							pthread_mutex_unlock(&mutex);

							break;

						case ENABLE_ACC:

							display_image_to_fb(acc, HOLD_IMAGE_X_SIZE, HOLD_IMAGE_Y_SIZE,
									HOLD_IMAGE_X_OFFSET, HOLD_IMAGE_Y_OFFSET);

							pthread_mutex_lock(&mutex);
							touch_enable=0;
							acc_enable=1;
							pthread_mutex_unlock(&mutex);

							break;

						case ENABLE_TOUCH:

							display_image_to_fb(touch, HOLD_IMAGE_X_SIZE, HOLD_IMAGE_Y_SIZE,
									HOLD_IMAGE_X_OFFSET, HOLD_IMAGE_Y_OFFSET);

							pthread_mutex_lock(&mutex);
							touch_enable=1;
							acc_enable=0;
							pthread_mutex_unlock(&mutex);

							break;
					}

					play_wav(hold_wav, HOLD_WAV_SIZE);

					break;

				default:
					
					break;
			}	
		}

		usleep(100000);
	}
}/*}}}*/

/*******************************************************************************************************
 *
 *  < void *acc_thread_func(void *arg) >
 *
 *  - 가속도계를 이용하여 동서남북 네방향의 기울임을 감지하여 해당 메뉴를 선택할 수 있다
 *
 *******************************************************************************************************/
void *acc_thread_func(void *arg)
{/*{{{*/
	int direction;
	int delay=50000;

	struct struct_accel *acc;
	struct struct_accel_data *data;
	
	//pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
	//pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);

	acc=acc_init(ACC_CAL_X, ACC_CAL_Y, ACC_CAL_Z);
	data=acc->struct_accel_data;
	
	while(1)
    	{	
#ifdef DEBUG
		printf("--------------------------\n%lf %lf %lf\n", data->dxv, data->dyv, data->dzv);
		printf("%d %d %d\n", data->rawx, data->rawy, data->rawz);
		printf("%lf %lf %lf\n", data->xtheta, data->ytheta, data->ztheta);
		printf("%lf %lf %lf\n\n", data->xacc, data->yacc, data->zacc);
#endif
		if(direction=check_acc(delay, ADC_ACCEL | ACC_MENU_DROP))
		{
			switch(direction)
			{
				case LEFT_CLICKED:

					if(acc_enable)
					{
						clicked_left();
						usleep(100000);
					}
					break;
				
				case RIGHT_CLICKED:

					if(acc_enable)
					{
						clicked_right();
						usleep(100000);
					}
					break;
				
				case UP_CLICKED:

					if( acc_enable && up_down_arrow_key )
					{
						clicked_up();
						usleep(100000);
					}
					break;
				
				case DOWN_CLICKED:
					
					if( acc_enable && up_down_arrow_key )
					{
						clicked_down();
						usleep(100000);
					}
					break;
			}
		}
	}
}/*}}}*/

/*******************************************************************************************************
 *
 *  < void *blue_thread_func(void *arg) >
 *
 * 호스트로 부터 들어오는 명령어를 블루투스를 통해 수신하여 처리한다.
 * 1. 테트리스 퍼즐 클라이언트 프로그램 실행
 * (추후 추가 예정)
 *
 *******************************************************************************************************/
void *blue_thread_func(void *arg)
{/*{{{*/
	struct blue_packet bt;
	unsigned int data[3]={ 0};
	
	bt_init();

	while(1)
	{
#ifdef DEBUG
		printf("blue while loop start\n");
#endif

		bt_recv(&bt, data, sizeof(int)*3);
#ifdef DEBUG
		printf("Recv : bt.ctl=[%d]", bt.ctl); 
		printf("t_blue_data[id:%d]", data[0]);
		printf("[race:%d]", data[1]);
		printf("[attack_damage:%d]\n", data[2]); 
#endif
		if( bt.ctl ==  BT_CTL_TET)
		{
			if( data == BT_CTL_TET_START)
			{
				pthread_mutex_lock(&mutex);
				t_puzzle_cli_exec=1;
				pthread_mutex_unlock(&mutex);
#ifdef DEBUG
				printf("t_puzzle_cli_exec = %d\n", t_puzzle_cli_exec);
#endif

			}
		}

		if(bt.ctl == BT_CTL_POCKETMON_CONTROL)
		{
			pthread_mutex_lock(&mutex);
			ctl_pocketmon_exec=1;
			pthread_mutex_unlock(&mutex);

#ifdef DEBUG
			printf("ctl_pocketmon_app_exec = %d\n", ctl_pocketmon_exec);
#endif
		}

	}
}/*}}}*/

/*********************************************************************************************************
 *
 *  < void *sudo_load_thread_func(void *arg) >
 *
 *  - 프로그램에 필요한 자원들을 로드하는 동한 로딩 이미지를 출력하기위한 함수
 *
 *********************************************************************************************************/
void *sudo_load_thread_func(void *arg)
{/*{{{*/
	int index=0;

	while(1)
	{
		display_image_to_fb(sudo_loading[index], SUDO_LOAD_IMAGE_X_SIZE, SUDO_LOAD_IMAGE_Y_SIZE,
				SUDO_LOAD_IMAGE_X_OFFSET, SUDO_LOAD_IMAGE_Y_OFFSET);

		if( index == 10)
			index=0;
		else
			index++;

		if( load_complete_flag == 1)
			break;

		usleep(100000);
	}

	pthread_exit(0);
}/*}}}*/

/********************************************************************************************************
 *
 *  < void up_down_arrow_lock(void) >
 *
 *  - 서브메뉴가 없는 메인 메뉴를 선택했을시 위, 아래 화살표를 잠근다.
 *
 ********************************************************************************************************/
void up_down_arrow_lock(void)
{/*{{{*/
	up_down_arrow_key=0;

	display_image_to_fb(arrow_hide, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			UP_ARROW_X_OFFSET, UP_ARROW_Y_OFFSET);
	display_image_to_fb(arrow_hide, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			DOWN_ARROW_X_OFFSET, DOWN_ARROW_Y_OFFSET);
}/*}}}*/

/********************************************************************************************************
 *
 *  < void up_down_arrow_unlock(void) >
 *
 *  - 서브메뉴가 있는 메인 메뉴를 선택했을시 위, 아래 화살표를 활성화 한다.
 *
 ********************************************************************************************************/
void up_down_arrow_unlock(void)
{/*{{{*/
	up_down_arrow_key=1;

	display_image_to_fb(up_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			UP_ARROW_X_OFFSET, UP_ARROW_Y_OFFSET);
	display_image_to_fb(down_arrow, ARROW_IMAGE_X_SIZE, ARROW_IMAGE_Y_SIZE,
			DOWN_ARROW_X_OFFSET, DOWN_ARROW_Y_OFFSET);
}/*}}}*/

/********************************************************************************************************
 *
 *  < void pause_menu(void) >
 *
 *  - 메뉴 선택을 통해 다른 프로그램을 실행시키기위해 메뉴 프로그램에서 사용된 쓰레드, 자원들을 해제한다.
 *
 ********************************************************************************************************/
void pause_menu(void)
{/*{{{*/

#ifdef DEBUG
	printf("pause_menu() start\n");
#endif
	pthread_cancel(ts_thread);
	pthread_join(ts_thread, &thread_result);
#ifdef DEBUG
	printf("ts_tread cancel\n");
#endif
	pthread_cancel(acc_thread);
	pthread_join(acc_thread, &thread_result);
#ifdef DEBUG
	printf("acc_thread cancel\n");
#endif
	pthread_cancel(blue_thread);
	pthread_join(blue_thread, &thread_result);
#ifdef DEBUG
	printf("blue_thread cancel\n");
#endif
	munmap(pfbdata, FB_SIZE*2);

	close(fb_fd);
	close(ts_fd);
	close(sound_fd);

	acc_exit();
	bt_exit();
#ifdef DEBUG
	printf("pause_menu() end\n");
#endif
}/*}}}*/

/*******************************************************************************************************
 *
 *  < void restart_menu(void) >
 *
 *  - 선택 되었던 프로그램이 종료되고 다시 메뉴 프로그램으로 복귀할 때 필요한 자원들을 생성해준다.
 *
 *******************************************************************************************************/
void restart_menu(void)
{/*{{{*/
	
	fb_fd=open(FB_DEV, O_RDWR);
	if( fb_fd < 0 )
    	{
		printf("%s open failed\n", FB_DEV);
		exit(-1);
    	}

    	ts_fd=open(TS_DEV, O_RDWR);
    	if( ts_fd < 0 )
    	{
		printf("%s open failed\n", TS_DEV);
		exit(-1);
    	}
	
	sound_fd=open(SOUND_DEV, O_WRONLY);
	if( sound_fd < 0 )
	{
		printf("%s open failed\n", SOUND_DEV);
		exit(-1);
	}

   	pfbdata=(unsigned short *) mmap(0, FB_SIZE*2, PROT_READ | PROT_WRITE, MAP_SHARED, fb_fd, 0);
	
	display_menu_image();
	
	pthread_create(&ts_thread, NULL, ts_thread_func, NULL);

	pthread_create(&acc_thread, NULL, acc_thread_func, NULL);

	pthread_create(&blue_thread, NULL, blue_thread_func, NULL);
}/*}}}*/

/**********************************************************************************************************
 * 
 * < void cleanup(void) >
 *
 * - 메뉴 프로그램과 관련된 자원 해제
 *
 **********************************************************************************************************/
void cleanup(void)
{/*{{{*/
	init_fb();

	munmap(pfbdata, FB_SIZE*2);

	close(fb_fd);
	close(ts_fd);
	close(sound_fd);

	acc_exit();
	bt_exit();

	exit(1);
}/*}}}*/
