#include "common.h"
#include "irda.h"
#include "neighbor.h"

/******************************************************************************************************
 * 
 * < main 함수 >
 *
 * 1. 프레임버퍼, 터치스크린, 사운드 장치 드라이버 오픈
 * 2. 프로그램 초기화, 프레임버퍼 초기화, IrDA초기화
 * 3. is_attached()함수를 이용하여 동서남북에서 접근된 모듈을 감지하여 해당하는 방향에 맞는 
 *    이미지를 보여준다.
 *    
 *******************************************************************************************************/ 
int main(int argc, char **argv)
{
	struct irda_instance irda;
	struct neighbor_instance neighbor[4];

	pthread_t ts_thread;
	void *thread_result;

	int i;

	fb_fd=open(FB_DEV, O_RDWR);
	if( fb_fd < 0 )
    	{
		printf("%s open failed\n", FB_DEV);
		exit(-1);
    	}

    	ts_fd=open(TS_DEV, O_RDWR);
    	if( ts_fd < 0 )
    	{
		printf("%s open failed\n", TS_DEV);
		exit(-1);
    	}
	
	sound_fd=open(SOUND_DEV, O_WRONLY);
    	if( sound_fd < 0 )
    	{
		printf("%s open failed\n", SOUND_DEV);
		exit(-1);
    	}
	
	memset(neighbor, 0, sizeof(struct neighbor_instance)*4);
	
	pfbdata=(unsigned short *) mmap(0, FB_SIZE*2, PROT_READ | PROT_WRITE, MAP_SHARED, fb_fd, 0);

	signal(SIGINT, cleanup);

	init_fb();

	init_loading_image();

	pthread_create(&load_thread, NULL, load_thread_func, NULL);
	
	init_neighbor_app();

	pthread_join(load_thread, &thread_result);

	display_neighbor_info_image();

	init_fb();

	display_neighbor_image();

	init_irda(&irda);

	pthread_create(&ts_thread, NULL, ts_thread_func, NULL);
	
	while(irda_key)
	{
		check_to_attach(&irda, neighbor);

		for(i=0; i<4; i++)
		{
			if(current_prio == 0)
			{
				display_image_to_fb(neighbor_front_a, NEIGHBOR_IMAGE_X_SIZE, NEIGHBOR_IMAGE_Y_SIZE,
						NEIGHBOR_IMAGE_X_OFFSET, NEIGHBOR_IMAGE_Y_OFFSET);
				break;
			}

			if(current_prio == neighbor[i].prio)
			{
				switch(i)
				{
					case DOWN_ATTACHED:
						display_image_to_fb(neighbor_down, NEIGHBOR_IMAGE_X_SIZE,
								NEIGHBOR_IMAGE_Y_SIZE, NEIGHBOR_IMAGE_X_OFFSET,
								NEIGHBOR_IMAGE_Y_OFFSET);

						printf("DOWN ATTACHED!\n");
						break;

					case LEFT_ATTACHED:
						display_image_to_fb(neighbor_left, NEIGHBOR_IMAGE_X_SIZE,
								NEIGHBOR_IMAGE_Y_SIZE, NEIGHBOR_IMAGE_X_OFFSET,
								NEIGHBOR_IMAGE_Y_OFFSET);

						printf("LEFT ATTACHED!\n");
						break;

					case UP_ATTACHED:
						display_image_to_fb(neighbor_up, NEIGHBOR_IMAGE_X_SIZE,
								NEIGHBOR_IMAGE_Y_SIZE, NEIGHBOR_IMAGE_X_OFFSET,
								NEIGHBOR_IMAGE_Y_OFFSET);

						printf("UP ATTACHED!\n");
						break;

					case RIGHT_ATTACHED:
						display_image_to_fb(neighbor_right, NEIGHBOR_IMAGE_X_SIZE,
								NEIGHBOR_IMAGE_Y_SIZE, NEIGHBOR_IMAGE_X_OFFSET,
								NEIGHBOR_IMAGE_Y_OFFSET);

						printf("RIGHT ATTACHED!\n");
						break;
				}
			}
		}

		if(power_off)
		{
			pthread_cancel(ts_thread);
			pthread_join(ts_thread, &thread_result);

			irda_close(&irda);
			
			cleanup();
		}

		usleep(100000);
	}

	irda_close(&irda);

	cleanup();
}

/***********************************************************************************
 *
 * < void init_neighbor_app(void) >
 *
 * - 이웃 프로그램에 필요한 이미지 파일과 사운드 파일을 메모리로 로드한다
 *
 ***********************************************************************************/
void init_neighbor_app(void)
{
	int fd;
	
	fd=open_image(POWER_IMAGE_PATH);
	copy_image_to_buff(fd, power, POWER_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(POWER_H_IMAGE_PATH);
	copy_image_to_buff(fd, power_h, POWER_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(INFO_IMAGE_PATH);
	copy_image_to_buff(fd, info, INFO_IMAGE_SIZE);
	close(fd);

	fd=open_image(INFO_H_IMAGE_PATH);
	copy_image_to_buff(fd, info_h, INFO_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_IMAGE_PATH);
	copy_image_to_buff(fd, stop, STOP_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_H_IMAGE_PATH);
	copy_image_to_buff(fd, stop_h, STOP_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(NEIGHBOR_MANUAL_IMAGE_PATH);
	copy_image_to_buff(fd, neighbor_manual, NEIGHBOR_MANUAL_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(NEIGHBOR_FRONT_A_IMAGE_PATH);
	copy_image_to_buff(fd, neighbor_front_a, NEIGHBOR_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(NEIGHBOR_FRONT_B_IMAGE_PATH);
	copy_image_to_buff(fd, neighbor_front_b, NEIGHBOR_IMAGE_SIZE);
	close(fd);

	fd=open_image(NEIGHBOR_LEFT_IMAGE_PATH);
	copy_image_to_buff(fd, neighbor_left, NEIGHBOR_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(NEIGHBOR_RIGHT_IMAGE_PATH);
	copy_image_to_buff(fd, neighbor_right, NEIGHBOR_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(NEIGHBOR_UP_IMAGE_PATH);
	copy_image_to_buff(fd, neighbor_up, NEIGHBOR_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(NEIGHBOR_DOWN_IMAGE_PATH);
	copy_image_to_buff(fd, neighbor_down, NEIGHBOR_IMAGE_SIZE);
	close(fd);
	
	fd=open_wav(POWER_WAV_PATH);
	copy_wav_to_buff(fd, power_wav, POWER_WAV_SIZE);
	close(fd);
	
	fd=open_wav(INFO_WAV_PATH);
	copy_wav_to_buff(fd, info_wav, INFO_WAV_SIZE);
	close(fd);

	fd=open_wav(CLICK_WAV_PATH);
	copy_wav_to_buff(fd, click_wav, CLICK_WAV_SIZE);
	close(fd);
	
	load_complete_flag=1;
}

/*********************************************************************************************
 *
 *  < void display_neighbor_image(void) >
 *
 *  - 이웃 프로그램과 관련된 초기 이미지 파일을 화면에 보여준다
 *
 *********************************************************************************************/
void display_neighbor_image(void)
{
	display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
			POWER_X_OFFSET, POWER_Y_OFFSET);

	display_image_to_fb(info, INFO_IMAGE_X_SIZE, INFO_IMAGE_Y_SIZE,
			INFO_IMAGE_X_OFFSET, INFO_IMAGE_Y_OFFSET);
	
	display_image_to_fb(neighbor_front_a, NEIGHBOR_IMAGE_X_SIZE, NEIGHBOR_IMAGE_Y_SIZE,
			NEIGHBOR_IMAGE_X_OFFSET, NEIGHBOR_IMAGE_Y_OFFSET);
}

/**********************************************************************************************
 *
 *  < void display_neighbor_info_image(void) >
 *
 *  - 사용자가 인포 버튼을 눌렀을 경우에 해당 도움말을 보여주고 터치 스크린을 통해
 *    도움말 인터페이스를 컨트롤 한다
 *
 *********************************************************************************************/    
void display_neighbor_info_image(void)
{
	int ret;
	short ts_buf[4];
		
	display_image_to_fb(neighbor_manual, NEIGHBOR_MANUAL_IMAGE_X_SIZE, NEIGHBOR_MANUAL_IMAGE_Y_SIZE,
				NEIGHBOR_MANUAL_IMAGE_X_OFFSET, NEIGHBOR_MANUAL_IMAGE_Y_OFFSET);
	
	display_image_to_fb(stop, STOP_IMAGE_X_SIZE, STOP_IMAGE_Y_SIZE,
			STOP_IMAGE_X_OFFSET, STOP_IMAGE_Y_OFFSET);

	while(1)
	{
		ret=read(ts_fd, ts_buf, sizeof(ts_buf));
		if( ret < 0 )
		{
	    		printf("Read Touch Screen Data Error\n");
			exit(-1);
		}

		ret=check_click_zone(ts_buf[1], ts_buf[2]);

		if( ts_buf[0] && ret == STOP )
		{

			display_image_to_fb(stop_h, STOP_IMAGE_X_SIZE, STOP_IMAGE_Y_SIZE,
					STOP_IMAGE_X_OFFSET, STOP_IMAGE_Y_OFFSET);
				
			play_wav(click_wav, CLICK_WAV_SIZE);

			break;
		}
	}
}

/************************************************************************************************************
 *
 *  < void *ts_thread_func(void *arg) >
 *
 *  -  터치 스크린 인터페이스 함수
 *  	POWER_OFF	: 프로그램 종료
 *  	INFO		: 도움말 표시
 *
 ************************************************************************************************************/
void *ts_thread_func(void *arg)
{
	int ret;
	short ts_buf[4];

	while(1)
	{
		if(read(ts_fd,ts_buf,sizeof(ts_buf)) < 0)
		{
	    		printf("Read Touch Screen Data Error\n");
	    		cleanup();
		}	
		
		printf("Press[%d] X[%d] Y[%d] MillSec[%d]\n", ts_buf[0], ts_buf[1], ts_buf[2], ts_buf[3]);

		ret=check_click_zone(ts_buf[1], ts_buf[2]);
		
		if( ts_buf[0] )
		{
			
			switch( ret )
			{
				case POWER_OFF:

					display_image_to_fb(power_h, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
						POWER_X_OFFSET, POWER_Y_OFFSET);
				
					play_wav(power_wav, POWER_WAV_SIZE);

					display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
						POWER_X_OFFSET, POWER_Y_OFFSET);

					power_off=1;

				case INFO:
	
					irda_key=0;

					display_image_to_fb(info_h, INFO_IMAGE_X_SIZE, INFO_IMAGE_Y_SIZE,
							INFO_IMAGE_X_OFFSET, INFO_IMAGE_Y_OFFSET);
					
					play_wav(info_wav, INFO_WAV_SIZE);

					display_neighbor_info_image();

					init_fb();

					display_neighbor_image();

					irda_key=1;
					
					break;

				default:

					break;
			}
		}
			
		usleep(100000);
	}
}

/*********************************************************************************************************
 *
 *  < int check_click_zone(short x, short, y) >
 *
 *  - 터치 스크린 디바이스로 부터 얻어온 x, y좌표 값을 받아 설정해 놓은 구역에 합당하면
 *    해당 리턴값을 반환한다
 *
 *********************************************************************************************************/    
int check_click_zone(short x, short y)
{
	if( x > 560 && x < 630 )
	{
		if( y > 16 && y < 86 )
			return POWER_OFF;
	}

	if( x > 16 && x < 86 )
	{
		if( y > 16 && y < 86 )
			return INFO;
	}

	if( x > 460 && x < 500 )
	{
		if( y > 320 && y < 360)
			return STOP;
	}

}

void check_to_attach(struct irda_instance *irda, struct neighbor_instance *neighbor)
{

	if( is_attached(irda, DOWN_CHECK) )
	{
		//neighbor[0].attached_flag=1;
		
		if( neighbor[0].attached_flag == 0 )
		{
			current_prio++;
			neighbor[0].attached_flag=1;
			neighbor[0].prio=current_prio;
		}
		
	}else 
	{
		//neighbor[0].attached_flag=0;
		
		if( neighbor[0].attached_flag == 1 )
		{
			neighbor[0].attached_flag=0;
			neighbor[0].prio=0;
			
			update_current_prio(neighbor);
		}
		
	}

	if( is_attached(irda, LEFT_CHECK) )
	{
		//neighbor[1].attached_flag=1;
		
		if( neighbor[1].attached_flag == 0 )
		{
			current_prio++;
			neighbor[1].attached_flag=1;
			neighbor[1].prio=current_prio;
		}
		
	}else 
	{
		//neighbor[1].attached_flag=0;
		
		if( neighbor[1].attached_flag == 1 )
		{
			neighbor[1].attached_flag=0;
			neighbor[1].prio=0;
			
			update_current_prio(neighbor);
		}
		
	}

	if( is_attached(irda, UP_CHECK) )
	{
		//neighbor[2].attached_flag=1;
		
		if( neighbor[2].attached_flag == 0 )
		{
			current_prio++;
			neighbor[2].attached_flag=1;
			neighbor[2].prio=current_prio;
		}
		

	}else 
	{
		//neighbor[2].attached_flag=0;
		
		if( neighbor[2].attached_flag == 1 )
		{
			neighbor[2].attached_flag=0;
			neighbor[2].prio=0;
			
			update_current_prio(neighbor);
		}
		
	}

	if( is_attached(irda, RIGHT_CHECK) )
	{
		//neighbor[3].attached_flag=1;
		
		if( neighbor[3].attached_flag == 0 )
		{
			current_prio++;
			neighbor[3].attached_flag=1;
			neighbor[3].prio=current_prio;
		}
		

	}else 
	{
		//neighbor[3].attached_flag=0;
		
		if( neighbor[3].attached_flag == 1 )
		{
			neighbor[3].attached_flag=0;
			neighbor[3].prio=0;
			
			update_current_prio(neighbor);
		}
		
	}
}

void update_current_prio(struct neighbor_instance *neighbor)
{
	int i;
	int temp=0;

	

	for(i=0; i<4 ; i++)
	{
		if( neighbor[i].prio > temp )
			temp=neighbor[i].prio;
	}

	current_prio=temp;
}

/**********************************************************************************************************
 * 
 * < void cleanup(void) >
 *
 * - 이웃 프로그램과 관련된 자원 해제
 *
 **********************************************************************************************************/
void cleanup(void)
{
	init_fb();

	munmap(pfbdata, FB_SIZE*2);

	close(fb_fd);
	close(ts_fd);
	close(sound_fd);

	exit(1);
}
