#include "common.h"
#include "adc.h"
#include "wind.h"

/******************************************************************************************************
 * 
 * < main 함수 >
 *
 * 1. 프레임버퍼, 터치스크린, 사운드 장치 드라이버 오픈
 * 2. 프로그램 초기화, 프레임버퍼 초기화
 * 3. 바람 감지를 위한 wind_thread 생성
 * 4. 블루투스를 통한 전등 시작 패킷 전송
 * 5. 터치 스크린을 통한 인터페이스
 * 	- POWER_OFF	: 프로그램 종료
 * 	- INFO		: 도움말 출력
 * 
 *******************************************************************************************************/
int main(int argc, char **argv)
{
	int ret;
	short ts_buf[4];

	pthread_t wind_thread;
	void *thread_result;

	fb_fd=open(FB_DEV, O_RDWR);
	if( fb_fd < 0 )
    	{
		printf("%s open failed\n", FB_DEV);
		exit(-1);
    	}

    	ts_fd=open(TS_DEV, O_RDWR);
    	if( ts_fd < 0 )
    	{
		printf("%s open failed\n", TS_DEV);
		exit(-1);
    	}
     	
	sound_fd=open(SOUND_DEV, O_RDWR);
    	if( sound_fd < 0 )
    	{
		printf("%s open failed\n", SOUND_DEV);
		exit(-1);
    	} 

	pfbdata=(unsigned short *) mmap(0, FB_SIZE*2, PROT_READ | PROT_WRITE, MAP_SHARED, fb_fd, 0);

	signal(SIGINT, cleanup);

	init_fb();

	init_loading_image();

	pthread_create(&load_thread, NULL, load_thread_func, NULL);

	init_wind_app();

	pthread_join(load_thread, &thread_result);

	display_wind_info_image();

	init_fb();

	display_wind_image();

	pthread_create(&wind_thread, NULL, wind_thread_func, NULL);
	
	while(1)
	{
		if(read(ts_fd,ts_buf,sizeof(ts_buf)) < 0)
		{
	    		printf("Read Touch Screen Data Error\n");
	    		cleanup();
		}	
		
		printf("Press[%d] X[%d] Y[%d] MillSec[%d]\n", ts_buf[0], ts_buf[1], ts_buf[2], ts_buf[3]);

		ret=check_click_zone(ts_buf[1], ts_buf[2]);
		
		if( ts_buf[0] )
		{
			
			switch( ret )
			{
				case POWER_OFF:

					display_image_to_fb(power_h, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
						POWER_X_OFFSET, POWER_Y_OFFSET);
				
					play_wav(power_wav, POWER_WAV_SIZE);

					pthread_cancel(wind_thread);
					pthread_join(wind_thread, &thread_result);

					display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
						POWER_X_OFFSET, POWER_Y_OFFSET);

					cleanup();

				case INFO:
					
					acc_key=0;

					display_image_to_fb(info_h, INFO_IMAGE_X_SIZE, INFO_IMAGE_Y_SIZE,
							INFO_IMAGE_X_OFFSET, INFO_IMAGE_Y_OFFSET);
					
					play_wav(info_wav, INFO_WAV_SIZE);

					display_wind_info_image();

					init_fb();

					display_wind_image();
					
					acc_key=1;

					break;

				default:

					break;
			}
		}
	
		usleep(100000);
	}
    
    	cleanup();
}

/***********************************************************************************
 *
 * < void init_wind_app(void) >
 *
 * - 바람 프로그램에 필요한 이미지 파일과 사운드 파일을 메모리로 로드한다
 *
 ***********************************************************************************/
void init_wind_app(void)
{
	int fd;
	
	fd=open_image(POWER_IMAGE_PATH);
	copy_image_to_buff(fd, power, POWER_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(POWER_H_IMAGE_PATH);
	copy_image_to_buff(fd, power_h, POWER_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(INFO_IMAGE_PATH);
	copy_image_to_buff(fd, info, INFO_IMAGE_SIZE);
	close(fd);

	fd=open_image(INFO_H_IMAGE_PATH);
	copy_image_to_buff(fd, info_h, INFO_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_IMAGE_PATH);
	copy_image_to_buff(fd, stop, STOP_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_H_IMAGE_PATH);
	copy_image_to_buff(fd, stop_h, STOP_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(WIND_MANUAL_IMAGE_PATH);
	copy_image_to_buff(fd, wind_manual, WIND_MANUAL_IMAGE_SIZE);
	close(fd);

	fd=open_image(WIND_IMAGE1_PATH);
	copy_image_to_buff(fd, wind_image1, WIND_IMAGE_SIZE);
	close(fd);

	fd=open_image(WIND_IMAGE2_PATH);
	copy_image_to_buff(fd, wind_image2, WIND_IMAGE_SIZE);
	close(fd);


	fd=open_wav(WIND_SCREAM_WAV_PATH);
	copy_wav_to_buff(fd, wind_scream_wav, WIND_SCREAM_WAV_SIZE);
	close(fd);
	
	fd=open_wav(POWER_WAV_PATH);
	copy_wav_to_buff(fd, power_wav, POWER_WAV_SIZE);
	close(fd);
	
	fd=open_wav(INFO_WAV_PATH);
	copy_wav_to_buff(fd, info_wav, INFO_WAV_SIZE);
	close(fd);

	fd=open_wav(CLICK_WAV_PATH);
	copy_wav_to_buff(fd, click_wav, CLICK_WAV_SIZE);
	close(fd);

	load_complete_flag=1;
}

/*********************************************************************************************
 *
 *  < void display_wind_image(void) >
 *
 *  - 바람 프로그램과 관련된 초기 이미지 파일을 화면에 보여준다
 *
 *********************************************************************************************/
void display_wind_image(void)
{
	display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
			POWER_X_OFFSET, POWER_Y_OFFSET);
	
	display_image_to_fb(info, INFO_IMAGE_X_SIZE, INFO_IMAGE_Y_SIZE,
			INFO_IMAGE_X_OFFSET, INFO_IMAGE_Y_OFFSET);

	display_image_to_fb(wind_image1, WIND_IMAGE_X_SIZE, WIND_IMAGE_Y_SIZE,
			WIND_IMAGE_X_OFFSET, WIND_IMAGE_Y_OFFSET);
}

/**********************************************************************************************
 *
 *  < void display_wind_info_image(void) >
 *
 *  - 사용자가 인포 버튼을 눌렀을 경우에 해당 도움말을 보여주고 터치 스크린을 통해
 *    도움말 인터페이스를 컨트롤 한다
 *
 *********************************************************************************************/ 
void display_wind_info_image(void)
{
	int ret;
	short ts_buf[4];
		
	display_image_to_fb(wind_manual, WIND_MANUAL_IMAGE_X_SIZE, WIND_MANUAL_IMAGE_Y_SIZE,
				WIND_MANUAL_IMAGE_X_OFFSET, WIND_MANUAL_IMAGE_Y_OFFSET);
	
	display_image_to_fb(stop, STOP_IMAGE_X_SIZE, STOP_IMAGE_Y_SIZE,
			STOP_IMAGE_X_OFFSET, STOP_IMAGE_Y_OFFSET);

	while(1)
	{
		ret=read(ts_fd, ts_buf, sizeof(ts_buf));
		if( ret < 0 )
		{
	    		printf("Read Touch Screen Data Error\n");
			exit(-1);
		}

		ret=check_click_zone(ts_buf[1], ts_buf[2]);

		if( ts_buf[0] && ret == STOP )
		{

			display_image_to_fb(stop_h, STOP_IMAGE_X_SIZE, STOP_IMAGE_Y_SIZE,
					STOP_IMAGE_X_OFFSET, STOP_IMAGE_Y_OFFSET);
				
			play_wav(click_wav, CLICK_WAV_SIZE);

			break;
		}
	}
}

/********************************************************************************************************
 *
 *  < void *wind_thread_func(void *arg) >
 *
 *  - 사용자가 피에조 필름 센서에 바람을 불게 되면 생기는 신호를 감지.
 *  - check_piezo()함수를 통해 센싱 값이 0이면 1을 리턴하여 화면의 이미지를 바꿔준다.
 *
 ********************************************************************************************************/
void *wind_thread_func(void *arg)
{
	struct struct_piezo *piezo;
	int cnt=1;
	
	piezo=piezo_init(0, 0xff0);

	while(1)
	{
		printf("piezo->raw = %d\n", piezo->raw);
		
		if(piezo->check_piezo(100000, ADC_PIEZO | PIEZO_WIND))
		{
			if(acc_key)
			{
				cnt++;
				if(cnt == 1)
				{
					display_image_to_fb(wind_image2, WIND_IMAGE_X_SIZE, WIND_IMAGE_Y_SIZE,
							WIND_IMAGE_X_OFFSET, WIND_IMAGE_Y_OFFSET);

					play_wav(wind_scream_wav, WIND_SCREAM_WAV_SIZE);

					display_image_to_fb(wind_image1, WIND_IMAGE_X_SIZE, WIND_IMAGE_Y_SIZE,
							WIND_IMAGE_X_OFFSET, WIND_IMAGE_Y_OFFSET);
				}
			}
		}else
			cnt=0;
	}
}

/*********************************************************************************************************
 *
 *  < int check_click_zone(short x, short, y) >
 *
 *  - 터치 스크린 디바이스로 부터 얻어온 x, y좌표 값을 받아 설정해 놓은 구역에 합당하면
 *    해당 리턴값을 반환한다
 *
 *********************************************************************************************************/  
int check_click_zone(short x, short y)
{
	if( x > 560 && x < 630 )
	{
		if( y > 16 && y < 86 )
			return POWER_OFF;
	}

	if( x > 16 && x < 86 )
	{
		if( y > 16 && y < 86 )
			return INFO;
	}

	if( x > 460 && x < 500 )
	{
		if( y > 320 && y < 360)
			return STOP;
	}

}

/**********************************************************************************************************
 * 
 * < void cleanup(void) >
 *
 * - 바람 프로그램과 관련된 자원 해제
 *
 **********************************************************************************************************/   
void cleanup(void)
{
	init_fb();

	munmap(pfbdata, FB_SIZE*2);

	close(fb_fd);
	close(ts_fd);
	close(sound_fd);

	piezo_exit();

	exit(1);
}
